function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume(key === "return" ? "return" : "next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _AwaitValue(value) { this.wrapped = value; }

function _asyncIterator(iterable) { var method; if (typeof Symbol !== "undefined") { if (Symbol.asyncIterator) { method = iterable[Symbol.asyncIterator]; if (method != null) return method.call(iterable); } if (Symbol.iterator) { method = iterable[Symbol.iterator]; if (method != null) return method.call(iterable); } } throw new TypeError("Object is not async iterable"); }

function _asyncGeneratorDelegate(inner, awaitWrap) { var iter = {}, waiting = false; function pump(key, value) { waiting = true; value = new Promise(function (resolve) { resolve(inner[key](value)); }); return { done: false, value: awaitWrap(value) }; } ; if (typeof Symbol === "function" && Symbol.iterator) { iter[Symbol.iterator] = function () { return this; }; } iter.next = function (value) { if (waiting) { waiting = false; return value; } return pump("next", value); }; if (typeof inner.throw === "function") { iter.throw = function (value) { if (waiting) { waiting = false; throw value; } return pump("throw", value); }; } if (typeof inner.return === "function") { iter.return = function (value) { if (waiting) { waiting = false; return value; } return pump("return", value); }; } return iter; }

import auth from 'solid-auth-client';
import { asList } from './util';
/**
 * A wrapper around a query engine
 * that uses Solid authenticated requests for updates.
 */

export default class SolidUpdateEngine {
  /**
   * Creates a wrapper around the given query engine.
   */
  constructor(sources, baseEngine) {
    // Preload source but silence errors; they will be thrown during execution
    this._source = this.getUpdateSource(sources);

    this._source.catch(() => null);

    this._engine = baseEngine;
  }
  /**
   * Creates an asynchronous iterable of results for the given SPARQL query.
   */


  execute(sparql, sources) {
    var _this = this;

    return _wrapAsyncGenerator(function* () {
      yield* _asyncGeneratorDelegate(_asyncIterator(/^\s*(?:INSERT|DELETE)/i.test(sparql) ? _this.executeUpdate(sparql, sources) : _this._engine.execute(sparql, sources)), _awaitAsyncGenerator);
    })();
  }
  /**
   * Creates an asynchronous iterable with the results of the SPARQL UPDATE query.
   */


  executeUpdate(sparql, sources) {
    let done = false;

    const next = async () => {
      if (done) return {
        done
      };
      done = true; // Send authenticated PATCH request to the document

      const source = await (sources ? this.getUpdateSource(sources) : this._source);
      const {
        ok,
        status,
        statusText
      } = await auth.fetch(source, {
        method: 'PATCH',
        headers: {
          'Content-Type': 'application/sparql-update'
        },
        body: sparql
      });
      if (!ok) throw new Error(`Update query failed (${status}): ${statusText}`); // Clear stale cached versions of the document

      await this.clearCache(source); // Return success

      return {
        value: asList({
          ok
        })
      };
    };

    return {
      next,
      return: noop,
      throw: noop,

      // required by the interface
      [Symbol.asyncIterator]() {
        return this;
      }

    };
  }
  /**
   * Parses the source(s) into the source to update.
   */


  async getUpdateSource(sources) {
    let source = await sources; // Transform URLs or terms into strings

    if (source instanceof URL) source = source.href;else if (source && typeof source.value === 'string') source = source.value; // Parse a string URL source

    if (typeof source === 'string') {
      if (!/^https?:\/\//.test(source)) throw new Error('Can only update an HTTP(S) document.');
      return source.replace(/#.*/, '');
    } // Flatten recursive calls to this function


    if (Array.isArray(source)) {
      source = await Promise.all(source.map(s => this.getUpdateSource(s)));
      source = [].concat(...source).filter(s => !!s);
      if (source.length !== 1) throw new Error('Can only update a single source.');
      return source[0];
    } // Error on unsupported sources


    throw new Error(`Unsupported source: ${source}`);
  }
  /**
   * Removes the given document (or all, if not specified) from the cache,
   * such that fresh results are obtained next time.
   */


  clearCache(document) {
    return this._engine.clearCache(document);
  }

}

function noop() {
  /* empty */
}